{ ****************************************************************** }
{                                                                    }
{   VCL component TSchSW                                             }
{                                                                    }
{   Serial checker + Generator                                       }
{                                                                    }
{   18 Aug 1999 at 12:28                                             }
{                                                                    }
{   Copyright  1999 by Snuki                                        }
{                                                                    }
{ ****************************************************************** }

unit uSchSW;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms;

type
  TMethodType = (smNormal, smPreSuff, smBack, smPair);
  TErrorCode  = (ecOK, ecNonExists, ecIO);

type
  TSchSW = class(TComponent)
    private
      FMethod : TMethodType;
      FOriSerial : String;
      FPrefix : String;
      FHiddenPrefix : String;
      FSW : Boolean;
      FSuffix : String;
      FHiddenSuffix : String;
      FUserName : String;
      FUserSerial : String;
      FRegFileName : String;

      FOnChangeSW : TNotifyEvent;
      FOnChangeSerial : TNotifyEvent;
      FOnChangeUserName : TNotifyEvent;

      procedure AutoInitialize;
      procedure AutoDestroy;

      function GetMethod : TMethodType;
      procedure SetMethod(Value : TMethodType);
      function GetPrefix : String;
      procedure SetPrefix(Value : String);
      function GetSuffix : String;
      procedure SetSuffix(Value : String);
      function GetHiddenPrefix : String;
      procedure SetHiddenPrefix(Value : String);
      function GetHiddenSuffix : String;
      procedure SetHiddenSuffix(Value : String);
      function GetUserName : String;
      procedure SetUserName(Value : String);
      function GetUserSerial : String;
      procedure SetUserSerial(Value : String);
      function GetRegFileName : String;
      procedure SetRegFileName(Value : String);

    protected
      property OriSerial : String read FOriSerial;

      procedure ChangeSW; virtual;
      procedure ChangeSerial; virtual;
      procedure ChangeUserName; virtual;

    public
      property SW : Boolean read FSW;

      procedure CheckSerial;
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      function GenerateSerial ( UName : string ) : string;
      function LoadReg : TErrorCode;
      function SaveReg : TErrorCode;

    published
      property OnChangeSW : TNotifyEvent
        read FOnChangeSW
        write FOnChangeSW;
      property OnChangeSerial : TNotifyEvent
        read FOnChangeSerial
        write FOnChangeSerial;
      property OnChangeUserName : TNotifyEvent
        read FOnChangeUserName
        write FOnChangeUserName;
      property Method : TMethodType
        read GetMethod
        write SetMethod
          default smPreSuff;
      property Prefix : String
        read GetPrefix
        write SetPrefix;
      property Suffix : String
        read GetSuffix
        write SetSuffix;
      property HiddenPrefix : String
        read GetHiddenPrefix
        write SetHiddenPrefix;
      property HiddenSuffix : String
        read GetHiddenSuffix
        write SetHiddenSuffix;
      property UserName : String
        read GetUserName
        write SetUserName;
      property UserSerial : String
        read GetUserSerial
        write SetUserSerial;
      property RegFileName : String
        read GetRegFileName
        write SetRegFileName;
  end;

procedure Register;

implementation

uses Crc32;

procedure Register;
begin
  RegisterComponents('Snuki', [TSchSW]);
end;

procedure TSchSW.AutoInitialize;
begin
  FMethod := smPreSuff;
  FPrefix := '';
  FSuffix := '';
  FHiddenPrefix := '';
  FHiddenSuffix := '';
  FUserName := 'DEMO';
  FUserSerial := '0';
  FRegFileName := '';
end;

procedure TSchSW.AutoDestroy;
begin
  { No objects from AutoInitialize to free }
end;

function TSchSW.GetMethod : TMethodType;
begin
  Result := FMethod;
end;

procedure TSchSW.SetMethod(Value : TMethodType);
begin
  FMethod := Value;
  CheckSerial;
end;

function TSchSW.GetPrefix : String;
begin
  Result := FPrefix;
end;

procedure TSchSW.SetPrefix(Value : String);
begin
  FPrefix := Value;
  CheckSerial;
end;

function TSchSW.GetSuffix : String;
begin
  Result := FSuffix;
end;

procedure TSchSW.SetSuffix(Value : String);
begin
  FSuffix := Value;
  CheckSerial;
end;

function TSchSW.GetHiddenPrefix : String;
begin
  Result := FHiddenPrefix;
end;

procedure TSchSW.SetHiddenPrefix(Value : String);
begin
  FHiddenPrefix := Value;
  CheckSerial;
end;

function TSchSW.GetHiddenSuffix : String;
begin
  Result := FHiddenSuffix;
end;

procedure TSchSW.SetHiddenSuffix(Value : String);
begin
  FHiddenSuffix := Value;
  CheckSerial;
end;

function TSchSW.GetUserName : String;
begin
  Result := FUserName;
end;

procedure TSchSW.SetUserName(Value : String);
begin
  FUserName := Value;
  CheckSerial;
  ChangeUserName;
end;

function TSchSW.GetUserSerial : String;
begin
  Result := FUserSerial;
end;

procedure TSchSW.SetUserSerial(Value : String);
begin
  FUserSerial := Value;
  CheckSerial;
  ChangeSerial;
end;

function TSchSW.GetRegFileName : String;
begin
  Result := FRegFileName;
end;

procedure TSchSW.SetRegFileName(Value : String);
begin
  FRegFileName := Value;
end;

procedure TSchSW.ChangeSW;
begin
  if Assigned(FOnChangeSW) then
    FOnChangeSW(Self)
end;

procedure TSchSW.ChangeSerial;
begin
  if Assigned(FOnChangeSerial) then
    FOnChangeSerial(Self)
end;

procedure TSchSW.ChangeUserName;
begin
  if Assigned(FOnChangeUserName) then
    FOnChangeUserName(Self)
end;

procedure TSchSW.CheckSerial;
var
  CRC32      :  DWORD;
  SWT, SWOLD :  boolean;
  SU         :  string;
  SS, S2     :  string;
  qc         :  integer;
begin
  su:='';
  ss:='';
  s2:='';
  case FMethod of
    smNormal  : ss := FUserName;
    smPreSuff : ss := FPrefix+FHiddenPrefix+FUserName+FHiddenSuffix+FSuffix;
    smBack    : begin
      s2 := FSuffix+FHiddenSuffix+FUserName+FHiddenPrefix+FPrefix;
      for qc:=1 to Length(s2) do
        ss:=ss+s2[Length(s2)-qc+1];
    end;
    smPair    : begin
      if Length(FPrefix)>0 then
      begin
        for qc:=1 to Length(FPrefix) do
          if (qc mod 2)=0 then ss:=ss+FPrefix[qc]
                          else s2:=s2+FPrefix[qc];
      end;
      if Length(FHiddenPrefix)>0 then
      begin
        for qc:=1 to Length(FHiddenPrefix) do
          if (qc mod 2)=0 then ss:=ss+FHiddenPrefix[qc]
                          else s2:=s2+FHiddenPrefix[qc];
      end;
      if Length(FUserName)>0 then
      begin
        for qc:=1 to Length(FUserName) do
          if (qc mod 2)=0 then ss:=ss+FUserName[qc]
                          else s2:=s2+FUserName[qc];
      end;
      if Length(FHiddenSuffix)>0 then
      begin
        for qc:=1 to Length(FHiddenSuffix) do
          if (qc mod 2)=0 then ss:=ss+FHiddenSuffix[qc]
                          else s2:=s2+FHiddenSuffix[qc];
      end;
      if Length(FSuffix)>0 then
      begin
        for qc:=1 to Length(FSuffix) do
          if (qc mod 2)=0 then ss:=ss+FSuffix[qc]
                          else s2:=s2+FSuffix[qc];
      end;
      ss:=ss+s2;
    end;
  end;

  CRC32 := $FFFFFFFF;    // To match PKZIP
  if LENGTH(ss) > 0    // Avoid access violation in D4
    then CalcCRC32 (Addr(ss[1]), LENGTH(ss), CRC32);

  FOriSerial:=IntToHex(CRC32,8);

  su:='';
  ss:='';
  s2:='';
  case FMethod of
    smNormal  : su := FOriSerial;
    smPreSuff : su := FPrefix+FOriSerial+FSuffix;
    smBack    : begin
      s2 := FOriSerial;
      for qc:=1 to Length(s2) do su:=su+s2[Length(s2)-qc+1];
      su:=FPrefix+su+FSuffix;
    end;
    smPair    : begin
      if Length(FOriSerial)>0 then
      begin
        for qc:=1 to Length(FOriSerial) do
          if (qc mod 2)=0 then su:=su+FOriSerial[qc]
                          else s2:=s2+FOriSerial[qc];
      end;
      su:=FPrefix+su+s2+FSuffix;
    end;
  end;

  swt:=not(su=FUserSerial);
  SWOld:=FSW;

  FSW:=swt;
  if SWOld<>swt then ChangeSW;
end;

constructor TSchSW.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoInitialize;
end;

destructor TSchSW.Destroy;
begin
  AutoDestroy;
  inherited Destroy;
end;

function TSchSW.GenerateSerial ( UName : string ) : string;
var
  CRC32 :  DWORD;
  STemp :  string;
  SU    :  string;
  SS, S2:  string;
  qc    :  integer;
begin
  su:='';
  ss:='';
  s2:='';
  case FMethod of
    smNormal  : ss := UName;
    smPreSuff : ss := FPrefix+FHiddenPrefix+UName+FHiddenSuffix+FSuffix;
    smBack    : begin
      s2 := FSuffix+FHiddenSuffix+UName+FHiddenPrefix+FPrefix;
      for qc:=1 to Length(s2) do
        ss:=ss+s2[Length(s2)-qc+1];
    end;
    smPair    : begin
      if Length(FPrefix)>0 then
      begin
        for qc:=1 to Length(FPrefix) do
          if (qc mod 2)=0 then ss:=ss+FPrefix[qc]
                          else s2:=s2+FPrefix[qc];
      end;
      if Length(FHiddenPrefix)>0 then
      begin
        for qc:=1 to Length(FHiddenPrefix) do
          if (qc mod 2)=0 then ss:=ss+FHiddenPrefix[qc]
                          else s2:=s2+FHiddenPrefix[qc];
      end;
      if Length(UName)>0 then
      begin
        for qc:=1 to Length(UName) do
          if (qc mod 2)=0 then ss:=ss+UName[qc]
                          else s2:=s2+UName[qc];
      end;
      if Length(FHiddenSuffix)>0 then
      begin
        for qc:=1 to Length(FHiddenSuffix) do
          if (qc mod 2)=0 then ss:=ss+FHiddenSuffix[qc]
                          else s2:=s2+FHiddenSuffix[qc];
      end;
      if Length(FSuffix)>0 then
      begin
        for qc:=1 to Length(FSuffix) do
          if (qc mod 2)=0 then ss:=ss+FSuffix[qc]
                          else s2:=s2+FSuffix[qc];
      end;
      ss:=ss+s2;
    end;
  end;

  CRC32 := $FFFFFFFF;    // To match PKZIP
  if LENGTH(ss) > 0    // Avoid access violation in D4
    then CalcCRC32 (Addr(ss[1]), LENGTH(ss), CRC32);

  STemp:=IntToHex(CRC32,8);

  su:='';
  ss:='';
  s2:='';
  case FMethod of
    smNormal  : su := STemp;
    smPreSuff : su := FPrefix+STemp+FSuffix;
    smBack    : begin
      s2 := STemp;
      for qc:=1 to Length(s2) do
        su:=su+s2[Length(s2)-qc+1];
      su:=FPrefix+su+FSuffix;
    end;
    smPair    : begin
      if Length(Stemp)>0 then
      begin
        for qc:=1 to Length(STemp) do
          if (qc mod 2)=0 then su:=su+STemp[qc]
                          else s2:=s2+STemp[qc];
      end;
      su:=FPrefix+su+s2+FSuffix;
    end;
  end;

  Result:=su;
end;

function TSchSW.LoadReg : TErrorCode;
var fff : TextFile;
    x1, x2 : string;
begin
  if not(FileExists(FRegFileName)) then
  begin
    result:=ecNonExists;
    exit;
  end;
  AssignFile(fff,FRegFileName);
  {$I-}
  Reset(fff);
  Readln(fff,x1);
  Readln(fff,x2);
  CloseFile(fff);
  {$I+}

  if IOResult<>0 then
  begin
    result:=ecIO;
    exit;
  end;

  if FUserName<>x1 then
  begin
    FUserName:=x1;
    ChangeUserName;
  end;

  if FUserSerial<>x2 then
  begin
    FUserSerial:=x2;
    ChangeSerial;
  end;

  CheckSerial;

  Result:=ecOK;
end;

function TSchSW.SaveReg : TErrorCode;
var fff : TextFile;
begin
  AssignFile(fff,FRegFileName);
  {$I-}
  ReWrite(fff);
  Writeln(fff,FUserName);
  Writeln(fff,FUserSerial);
  CloseFile(fff);
  {$I+}

  if IOResult<>0 then result:=ecIO
                 else result:=ecOK;
end;

end.
